package com.gic.plug.web.controller;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.constants.Constants;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.goods.api.util.Constant;
import com.gic.plug.web.strategy.init.StrategyInit;
import com.gic.plug.web.vo.StoreResourceVO;
import com.gic.search.business.api.constant.enums.OperateEnum;
import com.gic.store.dto.StoreWidgetDTO;
import com.gic.store.service.StoreWidgetApiService;
import com.gic.widget.screening.api.dto.EsScreeningChainDetailDTO;
import com.gic.widget.screening.api.dto.EsScreeningEnterpriseCategorySceneInfoDTO;
import com.gic.widget.screening.api.dto.EsScreeningTemplateDTO;
import com.gic.widget.screening.api.service.EsScreeningInitService;
import com.gic.widget.screening.api.service.EsScreeningTemplateService;
import org.apache.commons.lang.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

/**
 *
 */
@Controller
public class ScreeningController {

    private Logger LOGGER = LogManager.getLogger(ScreeningController.class);

    @Autowired
    private EsScreeningInitService esScreeningInitService;
    @Autowired
    private EsScreeningTemplateService esScreeningTemplateService;
    @Autowired
    private StrategyInit strategyInit;
    @Autowired
    private StoreWidgetApiService storeWidgetApiService;

    /**
     * @return
     * @throws
     * @Title: getScreeningTemplate
     * @Description: 获取模板
     * @author xugh
     */
    @RequestMapping("get-screening-template")
    @ResponseBody
    public RestResponse getScreeningTemplate() {
        List<EsScreeningTemplateDTO> list = this.esScreeningTemplateService.queryEsScreeningTemplateList();
        return RestResponse.success(list);
    }

    /**
     * @param sceneCode
     * @return
     * @throws
     * @Title: getScreeningInitData
     * @Description: 获取初始化筛选条件
     * @author xugh
     */
    @RequestMapping("get-screening-init-data")
    @ResponseBody
    public RestResponse getScreeningInitData(@RequestParam(defaultValue = "member") String sceneCode) {
        String enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId().toString();
        if (Constants.STORE_SCENECODE.equals(sceneCode)) {
            enterpriseId = Constants.INDEX_ENTERPRISEID;
        }
        if (Constant.ERP_GOODS_SCREEN.equals(sceneCode) || Constant.WEIMOB_GOODS_SCREEN.equals(sceneCode)) {
            //TODO
            enterpriseId = "goods-test-1129";
        }
        List<EsScreeningEnterpriseCategorySceneInfoDTO> list = this.esScreeningInitService
                .queryEsScreeningByEnterpriseAndScene(enterpriseId, sceneCode);
        Collections.sort(list, new Comparator<EsScreeningEnterpriseCategorySceneInfoDTO>() {
            @Override
            public int compare(EsScreeningEnterpriseCategorySceneInfoDTO o1, EsScreeningEnterpriseCategorySceneInfoDTO o2) {
                return o1.getCreateTime().compareTo(o2.getCreateTime());
            }
        });
        List<EsScreeningChainDetailDTO> all = new ArrayList<>();
        for (EsScreeningEnterpriseCategorySceneInfoDTO dto : list) {
            String categoryId = dto.getSceneCategoryId();
            EsScreeningChainDetailDTO item = this.esScreeningInitService.findRootChainByCategoryId(categoryId);
            all.add(item);
        }
        return RestResponse.success(all);
    }

    @RequestMapping("get-screening-data")
    @ResponseBody
    public RestResponse getScreeningData(String widgetChainId) {
        List<EsScreeningChainDetailDTO> list = this.esScreeningInitService
                .queryEsScreeningByParentChainId(widgetChainId);
        return RestResponse.success(list);
    }

    @RequestMapping("get-screening-chain-detail")
    @ResponseBody
    public RestResponse getScreeningChainDetail(String parentChainId, String widgetFieldKey) {
        EsScreeningChainDetailDTO dto = this.esScreeningInitService.findChainDetail(parentChainId, widgetFieldKey);
        return RestResponse.success(dto);
    }


    @RequestMapping("get-screening-widget-chain-detail")
    @ResponseBody
    public Object getScreeningWidgetDetail(String widgetChainId) {
        EsScreeningChainDetailDTO dto = this.esScreeningInitService.findChainDetail(widgetChainId);
        return RestResponse.success(dto);
    }

    @RequestMapping("screen-back")
    @ResponseBody
    public Object screenBack(String param) throws ClassNotFoundException {
        return RestResponse.success(this.getScreenBack(param));
    }

    @RequestMapping("get-user-store-resource")
    @ResponseBody
    public RestResponse getUserStoreResouce() throws ClassNotFoundException {
        if(UserDetailUtils.getUserDetail().getUserInfo().getSuperAdmin() != 1){
            Long storeResource = UserDetailUtils.getUserDetail().getUserResourceInfo().getStoreResource();
            if(storeResource == null){
                return RestResponse.success(new StoreResourceVO());
            }
            ServiceResponse<StoreWidgetDTO> storeWidget = this.storeWidgetApiService.getStoreWidget(storeResource.intValue());
            if(storeWidget.isSuccess() && storeWidget.getResult() != null){
                StoreResourceVO vo = new StoreResourceVO();
                vo.setAuthMode(storeWidget.getResult().getAuthMode());
                vo.setSearchJson(storeWidget.getResult().getSearchParam());
                vo.setSceenBack(this.getScreenBack(storeWidget.getResult().getSearchParam()));
                return RestResponse.success(vo);
            }
        }
        return RestResponse.success(new StoreResourceVO());
    }

    private String getScreenBack(String param) throws ClassNotFoundException {
        if(StringUtils.isBlank(param)){
            return "";
        }
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        JSONArray array = JSON.parseArray(param);
        StringBuilder backMessage = new StringBuilder();
        for(int i=0; i< array.size(); i++){
            JSONObject jsonObject = array.getJSONObject(i);
            String type = jsonObject.getString("type");
            if(StringUtils.isNotBlank(type)){
                if(OperateEnum.OPERATE_AND_NOT.getValue().equals(type)){
                    backMessage.append(OperateEnum.OPERATE_AND_NOT.getDescribe()).append(" ");
                }
                if(OperateEnum.OPERATE_OR.getValue().equals(type)){
                    backMessage.append(OperateEnum.OPERATE_OR.getDescribe()).append(" ");
                }
            }
            JSONArray list = jsonObject.getJSONArray("list");
            StringBuilder stringBuilder = new StringBuilder();
            for(int j = 0; j<list.size(); j++){
                JSONObject json = list.getJSONObject(j);
                String key = json.getJSONObject("data").getString("key");
                String value = json.getJSONObject("data").getString("value");
                String compute = json.getJSONObject("data").getString("compute");
                String message = this.strategyInit.getStrategyMap().get(key).screenBack(compute, value, key, enterpriseId);
                stringBuilder.append(message);
                if(j < list.size()-1){
                    stringBuilder.append("并且");
                }
            }
            backMessage.append(stringBuilder);
        }
        return backMessage.toString();
    }
}
