package com.gic.plug.web.controller;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.plug.web.qo.StoreWidgetCountQO;
import com.gic.plug.web.qo.StoreWidgetQO;
import com.gic.plug.web.vo.StoreRegionVO;
import com.gic.plug.web.vo.StoreVO;
import com.gic.store.constant.StoreESFieldsEnum;
import com.gic.store.dto.StoreInfoDTO;
import com.gic.store.dto.StoreRegionDTO;
import com.gic.store.dto.StoreSearchDTO;
import com.gic.store.dto.StoreWidgetDTO;
import com.gic.store.service.StoreApiService;
import com.gic.store.service.StoreRegionApiService;
import com.gic.store.service.StoreWidgetApiService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * @author zhiwj
 * @date 2019/7/12
 */
@RestController
public class StoreController {

    @Autowired
    private StoreRegionApiService storeRegionApiService;

    @Autowired
    private StoreApiService storeApiService;
    @Autowired
    private StoreWidgetApiService storeWidgetApiService;

    @RequestMapping("/list-store-region")
    public RestResponse listStoreRegion(String search) {
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        ServiceResponse<List<StoreRegionDTO>> serviceResponse = storeRegionApiService.listStoreRegion(enterpriseId, search);
        if (serviceResponse.isSuccess()) {
            List<StoreRegionDTO> result = serviceResponse.getResult();
            return RestResponse.success(EntityUtil.changeEntityListByOrika(StoreRegionVO.class, result));
        } else {
            return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
    }

    @RequestMapping("/list-part-store")
    public RestResponse listPartStore(Integer storeGroupId, String search) {
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        StoreSearchDTO storeSearchDTO = new StoreSearchDTO();
        storeSearchDTO.setEnterpriseId(enterpriseId);
        storeSearchDTO.setStoreGroupId(storeGroupId);
        storeSearchDTO.setSearch(search);
        if(UserDetailUtils.getUserDetail().getUserResourceInfo() != null){
            storeSearchDTO.setStoreResource(UserDetailUtils.getUserDetail().getUserResourceInfo().getStoreResource());
        }
        String returnFields = StoreESFieldsEnum.STOREID.getField()+","+StoreESFieldsEnum.STORECODE.getField()+","+StoreESFieldsEnum.STORENAME.getField();
        ServiceResponse serviceResponse = storeApiService.listStore(storeSearchDTO,
                1, 100, returnFields);
        if (serviceResponse.isSuccess()) {
            return RestResponse.success(serviceResponse.getResult());
        } else {
            return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
    }

    @RequestMapping("get-store-count")
    public RestResponse getStoreCount(@RequestBody StoreWidgetCountQO storeWidgetCountQO){
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        StoreSearchDTO storeSearchDTO = new StoreSearchDTO();
        storeSearchDTO.setEnterpriseId(enterpriseId);
        storeSearchDTO.setSearchJson(storeWidgetCountQO.getSearchJson());
        storeSearchDTO.setAuthSearchJson(storeWidgetCountQO.getAuthSearchJson());
        ServiceResponse<Long> response = storeApiService.queryStoreCountFromES(storeSearchDTO);
        return RestResponse.success(response.getResult());
    }

    @RequestMapping("save-store-widget")
    public RestResponse saveStoreWidget(@RequestBody StoreWidgetQO storeWidgetQO){
        StoreWidgetDTO storeWidgetDTO = new StoreWidgetDTO();
        storeWidgetDTO.setAuthSearchParam(storeWidgetQO.getAuthSearchParam());
        storeWidgetDTO.setSearchParam(storeWidgetQO.getSearchParam());
        storeWidgetDTO.setStoreWidgetId(storeWidgetQO.getStoreWidgetId());
        storeWidgetDTO.setWriteBackParam(storeWidgetQO.getWriteBackParam());
        ServiceResponse<Integer> storeWidgetResponse = this.storeWidgetApiService.saveAuthStoreWidget(storeWidgetDTO);
        if(storeWidgetResponse.isSuccess()){
            return RestResponse.success(storeWidgetResponse.getResult());
        }
        return RestResponse.success();
    }

    @RequestMapping("get-store-widget")
    public RestResponse getStoreWidget(Integer storeWidgetId){
        ServiceResponse<StoreWidgetDTO> response = this.storeWidgetApiService.getStoreWidget(storeWidgetId);
        return RestResponse.success(response.getResult());
    }
}
