package com.gic.plug.web.controller.goods;

import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.goods.api.dto.GoodsCordDTO;
import com.gic.goods.api.dto.SpecCordDTO;
import com.gic.goods.api.service.CordApiService;
import com.gic.goods.api.service.MallGoodsBusinessApiService;
import com.gic.goods.api.util.Constant;
import com.gic.mall.base.api.enums.ErrorCodeEnums;
import com.gic.mall.share.api.dto.shop.ShopDTO;
import com.gic.mall.share.api.service.ShopApiService;
import com.gic.member.config.api.dto.AppletsConfigSimpleOutputDTO;
import com.gic.member.config.api.service.AppletsConfigApiService;
import com.gic.plug.web.vo.goods.MallGoodsCordVO;
import com.gic.search.business.api.constant.enums.OperateEnum;
import com.gic.search.business.api.dto.DynamicSearchDTO;
import com.gic.search.business.api.dto.ESResponseQueryBatchDTO;
import com.gic.search.business.api.service.EsBusinessOperaApiService;
import com.gic.search.business.api.utils.QueryConditionAssemblyUtil;
import com.gic.widget.screening.api.service.EsScreeningSearchService;
import com.google.common.base.Joiner;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 商城装修选择器
 */
@RestController
public class AppletConfigGoodsController {

    @Autowired
    private CordApiService cordApiService;

    @Autowired
    private ShopApiService shopApiService;

    @Autowired
    private AppletsConfigApiService appletsConfigApiService;

    @Autowired
    private EsScreeningSearchService esScreeningSearchService;

    @Autowired
    private EsBusinessOperaApiService esBusinessOperaApiService;

    @Autowired
    private MallGoodsBusinessApiService mallGoodsBusinessApiService;


    private static final Logger log = LogManager.getLogger(AppletConfigGoodsController.class);

    @RequestMapping("get-mall-cord-list")
    public RestResponse getMallCordList(String appConfigId, Long shopId) {
        Long areaId = null;
        if (StringUtils.isNotBlank(appConfigId)) {
            ServiceResponse<AppletsConfigSimpleOutputDTO> appServiceResponse = appletsConfigApiService.selectAppletsConfigIdByAppId(appConfigId, 1129);
            if (appServiceResponse.getResult() == null) {
                return RestResponse.failure(ErrorCodeEnums.MAll_ERR.getCode(), "小程序不存在");
            }
            ServiceResponse<ShopDTO> shopDTOServiceResponse = shopApiService.getShopByAppletId(appServiceResponse.getResult().getAppletConfigId(), 1129, 1);
            if (shopDTOServiceResponse.getResult() == null) {
                return RestResponse.failure(ErrorCodeEnums.MAll_ERR.getCode(), "不存在店铺");
            }
            areaId = shopDTOServiceResponse.getResult().getAreaId();
        } else if (shopId != null) {
            ServiceResponse<ShopDTO> shopDTOServiceResponse = shopApiService.getShopDetailsById(shopId, 1129);
            if (shopDTOServiceResponse.getResult() == null) {
                return RestResponse.failure(ErrorCodeEnums.MAll_ERR.getCode(), "不存在店铺");
            }
            areaId = shopDTOServiceResponse.getResult().getAreaId();
        }
        ServiceResponse<List<SpecCordDTO>> listServiceResponse = cordApiService.listSpecCordDTO(areaId);
        return RestResponse
                .success(EntityUtil.changeEntityListByOrika(MallGoodsCordVO.class, listServiceResponse.getResult()));
    }

    /**
     * 商城组对外提供的组件
     * @param currentPage
     * @param pageSize
     * @param appConfigId
     * @param search
     * @param cordId
     * @return
     */
    @RequestMapping("list-mall-goods-selector")
    public RestResponse listGoodsByChannel(@RequestParam(value = "currentPage", defaultValue = "1") Integer currentPage,
            @RequestParam(value = "pageSize", defaultValue = "20") Integer pageSize, String appConfigId, String search, Long cordId,
                                           Long shopId) {
        Long areaId = null;
        if (StringUtils.isNotBlank(appConfigId)) {
            ServiceResponse<AppletsConfigSimpleOutputDTO> appServiceResponse = appletsConfigApiService.selectAppletsConfigIdByAppId(appConfigId, 1129);
            if (appServiceResponse.getResult() == null) {
                return RestResponse.failure(ErrorCodeEnums.MAll_ERR.getCode(), "小程序不存在");
            }
            Integer shopType = 1;
            ServiceResponse<ShopDTO> shopDTOServiceResponse = shopApiService.getShopByAppletId(appServiceResponse.getResult().getAppletConfigId(), 1129, shopType);
            if (shopDTOServiceResponse.getResult() == null) {
                return RestResponse.failure(ErrorCodeEnums.MAll_ERR.getCode(), "不存在店铺");
            }
            areaId = shopDTOServiceResponse.getResult().getAreaId();
        } else if (shopId != null) {
            ServiceResponse<ShopDTO> shopDTOServiceResponse = shopApiService.getShopDetailsById(shopId, 1129);
            if (shopDTOServiceResponse.getResult() == null) {
                return RestResponse.failure(ErrorCodeEnums.MAll_ERR.getCode(), "不存在店铺");
            }
            areaId = shopDTOServiceResponse.getResult().getAreaId();
        }
        JSONObject cordJSON = null;
        if (cordId != null) {
            ServiceResponse<List<GoodsCordDTO>> serviceResponse = cordApiService.listChildAll(cordId);
            List<Long> cordList = new ArrayList<>();
            for (GoodsCordDTO goodsCord : serviceResponse.getResult()) {
                cordList.add(goodsCord.getSpecCordId());
            }
            cordList.add(cordId);
            cordJSON = QueryConditionAssemblyUtil.createSimpleQueryNode("cordId",
                    OperateEnum.OPERATE_CONTAIN, Joiner.on(" ").join(cordList));
        }
        ServiceResponse<ESResponseQueryBatchDTO> serviceResponse = searchGoods(currentPage, pageSize, areaId, search,
                Constant.CHANNEL_CODE_MALL, cordJSON);
        if (serviceResponse.isSuccess()) {
                List<Long> goodsIds = new ArrayList<>();
                if (CollectionUtils.isNotEmpty(serviceResponse.getResult().getRes())) {
                    serviceResponse.getResult().getRes().forEach(json -> {
                        Long goodsId = json.getLong("goodsId");
                        goodsIds.add(goodsId);
                    });
                }
                ServiceResponse<Map<Long, BigDecimal>> mapServiceResponse = mallGoodsBusinessApiService.getLowestSkuPrice(goodsIds, 1129);
                if (mapServiceResponse.isSuccess()) {
                    Map<Long, BigDecimal> map = mapServiceResponse.getResult();
                    serviceResponse.getResult().getRes().forEach(json -> {
                        Long goodsId = json.getLong("goodsId");
                        json.put("minSkuPrice", map.getOrDefault(goodsId, BigDecimal.valueOf(0)));
                    });
                }
            return RestResponse.success(serviceResponse.getResult());
        }
        return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
    }

    /**
     * 商城组对内的渠道商品组件
     * @param currentPage
     * @param pageSize
     * @param search
     * @return
     */
    @RequestMapping("list-channel-goods-selector")
    public RestResponse listGoodsChannel(@RequestParam(value = "currentPage", defaultValue = "1") Integer currentPage,
                                           @RequestParam(value = "pageSize", defaultValue = "20") Integer pageSize, String search,
                                            Double startPrice, Double endPrice, Integer startStock, Integer endStock, String channelCode) {
        JSONObject queryObject = null;
        if (startPrice != null) {
            queryObject = QueryConditionAssemblyUtil.createSimpleQueryNode("goodsPrice",
                    OperateEnum.OPERATE_GTE, startPrice);
        }
        if (endPrice != null) {
            if (queryObject == null) {
                queryObject = QueryConditionAssemblyUtil.createSimpleQueryNode("goodsPrice",
                        OperateEnum.OPERATE_LTE, endPrice);
            } else {
                queryObject = QueryConditionAssemblyUtil.addQueryNodeForSiblin("goodsPrice",
                        OperateEnum.OPERATE_LTE, endPrice, queryObject, OperateEnum.OPERATE_AND);
            }
        }
        if (startStock != null) {
            if (queryObject == null) {
                queryObject = QueryConditionAssemblyUtil.createSimpleQueryNode("stock",
                        OperateEnum.OPERATE_GTE, startStock);
            } else {
                queryObject = QueryConditionAssemblyUtil.addQueryNodeForSiblin("stock",
                        OperateEnum.OPERATE_GTE, startStock, queryObject, OperateEnum.OPERATE_AND);
            }
        }
        if (endStock != null) {
            if (queryObject == null) {
                queryObject = QueryConditionAssemblyUtil.createSimpleQueryNode("stock",
                        OperateEnum.OPERATE_LTE, endStock);
            } else {
                queryObject = QueryConditionAssemblyUtil.addQueryNodeForSiblin("stock",
                        OperateEnum.OPERATE_LTE, endStock, queryObject, OperateEnum.OPERATE_AND);
            }
        }
        ServiceResponse<ESResponseQueryBatchDTO> serviceResponse = searchGoods(currentPage, pageSize, null, search,
                Constant.CHANNEL_CODE_MALL, queryObject);
        if (serviceResponse.isSuccess()) {
            return RestResponse.success(serviceResponse.getResult());
        }
        return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
    }

    private ServiceResponse<ESResponseQueryBatchDTO> searchGoods(Integer currentPage, Integer pageSize,
            Long goodsDomainId, String search, String channelCode, JSONObject queryObject) {
        DynamicSearchDTO searchDTO = new DynamicSearchDTO();
        String searchCode = "";
        if (Constant.CHANNEL_CODE_MALL.equals(channelCode)) {
            searchCode = "gic_mall_goods";
        } else if (Constant.CHANNEL_CODE_ERP.equals(channelCode)) {
            searchCode = "erp_goods";
        }
        String esName = esScreeningSearchService.getCurrentEsName("1129", searchCode);
        searchDTO.setIndexName(esName);
        searchDTO.setColumnCategoryCode(searchCode);
        searchDTO.setEnterpriseId("1129");
        searchDTO.setType("mapper_type");
        searchDTO.setBegin((currentPage - 1) * pageSize);
        searchDTO.setRecordNumber(pageSize);
        JSONObject jsonObject = null;
        JSONObject status = QueryConditionAssemblyUtil.createSimpleQueryNode("status", OperateEnum.OPERATE_EQ, 2);
        if (Constant.CHANNEL_CODE_MALL.equals(channelCode)) {
            status = QueryConditionAssemblyUtil.addQueryNodeForSiblin(status, QueryConditionAssemblyUtil.createSimpleQueryNode("upShelfTime", OperateEnum.OPERATE_LTE, new Date().getTime()), OperateEnum.OPERATE_AND);
        }
        if (goodsDomainId == null) {
            jsonObject = status;
        } else {
            jsonObject = QueryConditionAssemblyUtil.createSimpleQueryNode("goodsDomainId",
                    OperateEnum.OPERATE_EQ, goodsDomainId);
            jsonObject = QueryConditionAssemblyUtil.addQueryNodeForSiblin(status, jsonObject, OperateEnum.OPERATE_AND);
        }
        if (StringUtils.isNotBlank(search)) {
            JSONObject goodsCodeJson = QueryConditionAssemblyUtil.createSimpleQueryNode("goodsCode",
                    OperateEnum.OPERATE_LIKE, search);
            JSONObject goodsNameJson = QueryConditionAssemblyUtil.createSimpleQueryNode("goodsName",
                    OperateEnum.OPERATE_LIKE, search);
            JSONObject goodsBarCodeJson = QueryConditionAssemblyUtil.createSimpleQueryNode("goodsBarCode",
                    OperateEnum.OPERATE_LIKE, search);
            List<JSONObject> jsonArray = new ArrayList<>();
            jsonArray.add(goodsCodeJson);
            jsonArray.add(goodsNameJson);
            jsonArray.add(goodsBarCodeJson);
            JSONObject object = QueryConditionAssemblyUtil.createListQueryNode(jsonArray, OperateEnum.OPERATE_OR);
            jsonObject = QueryConditionAssemblyUtil.addQueryNodeForSiblin(object, jsonObject, OperateEnum.OPERATE_AND);
        }
        if (queryObject != null) {
            jsonObject = QueryConditionAssemblyUtil.addQueryNodeForSiblin(queryObject, jsonObject, OperateEnum.OPERATE_AND);
        }
        searchDTO.setSearchJson(jsonObject);
        ServiceResponse<ESResponseQueryBatchDTO> serviceResponse = esBusinessOperaApiService.queryDataBatch(searchDTO,
                false, null);
        return serviceResponse;
    }

}
